#!/usr/bin/python3
"""
Date: 12/02/2021

Author: Marcely Zanon Boito

Usage: python3 get_json_stats.py <json file>
       For example, if you want to get the stats of the json file "all_files.json" in the "jsons_and_stats" folder, 
       you will have to type from the archive "python3 scripts/get_json_stats.py all_files.json"
"""

import sys, copy
import datetime
from utils import load_json

def format_number(duration):
    hours = int(duration / 3600)
    minutes = int((duration % 3600) / 60)
    seconds = int((duration % 3600) % 60)
    return '{:02d}:{:02d}:{:02d}'.format(hours, minutes, seconds)
    #return datetime.timedelta(seconds=number)

def get_gender_count(data):
    gender_dict = {"M":0,"F":0, "U":0}
    for key in data:
        gender_dict[data[key]["spk_gender"]] += 1
    return gender_dict

def get_speakers(data):
    ids_dict = dict()
    ids_dict['M'] = list()
    ids_dict['F'] = list()
    ids_dict['U'] = list()
    for key in data:
        gender = data[key]["spk_gender"]
        spk_id = data[key]["spk_id"]
        if not spk_id in ids_dict[gender]:
            ids_dict[gender].append(spk_id)
    return ids_dict

def get_entries_per_gender(data, gender):
    gender_dict = copy.deepcopy(data)
    for s_key in data:
        if data[s_key]["spk_gender"] != gender:
            del gender_dict[s_key]
    return gender_dict

def get_key(data, key):
    return [float(data[s_key][key]) for s_key in data]

def get_total_duration(data):
    return sum(get_key(data,"duration"))

def get_average_duration(data):
    return sum(get_key(data,"duration"))/len(data)

def get_max_duration(data):
    return max(get_key(data,"duration"))

def get_min_duration(data):
    return min(get_key(data,"duration"))

def get_over_threshold(data, threshold=30):
    count = 0.0 
    for s_key in data:
        if float(data[s_key]["duration"]) > threshold:
            count += 1
    return count

def get_durations_from_data(data):
    total_duration = str(format_number(get_total_duration(data)))
    avg_duration = str(format_number(get_average_duration(data)))
    min_duration = str(format_number(get_min_duration(data)))
    max_duration = str(format_number(get_max_duration(data)))
    return [total_duration, avg_duration, min_duration, max_duration]

def print_stats(data):
    gender_dict = get_gender_count(data)
    gender_ids = get_speakers(data)
    for key in gender_dict:
        if gender_dict[key] > 0:
            gender_data = get_entries_per_gender(data,key)
            print("GENDER ", key)
            print("Number of utterances:", gender_dict[key])
            print("Number of speakers:", len(gender_ids[key]))
            print()
            total_duration, avg_duration, min_duration, max_duration = get_durations_from_data(gender_data)
            print("Utterances length summary for gender", key)
            print("\t total duration:", total_duration)
            print("\t average:", avg_duration)
            print("\t min:", min_duration)
            print("\t max:", max_duration)
            print()
    print("TOTAL")
    print()
    print("Number of utterances:", len(data))
    print("Number of speakers:", len(gender_ids["M"])+len(gender_ids["F"])+len(gender_ids["U"])) 
    total_duration, avg_duration, min_duration, max_duration = get_durations_from_data(data)
    print("Utterances length summary for ALL")
    print("\t total duration:", total_duration)
    print("\t average:", avg_duration)
    print("\t min:", min_duration)
    print("\t max:", max_duration)
    print()

def get_empty_line(key):
    return ",".join([key,"0",str(format_number(0)), str(format_number(0)),str(format_number(0)), str(format_number(0))])

def get_header_line():
    return ",".join(["Speakers", "Number of Utterances", "Number of Speakers", "Total Duration", "Avg. Duration", "Min. Duration", "Max. Duration"])

def write_stats(data, file_name):
    with open(file_name, "w") as output_file:
        line = get_header_line()
        output_file.write(line + "\n")
        gender_dict = get_gender_count(data)
        ids_dict = get_speakers(data)
        for key in gender_dict:
            if gender_dict[key] > 0:
                utterances_number = str(gender_dict[key])
                gender_data = get_entries_per_gender(data,key)
                line = ",".join([key, utterances_number, str(len(ids_dict[key]))] + get_durations_from_data(gender_data))
            else:
                line = get_empty_line(key)
            output_file.write(line + "\n")
        line = ",".join(["ALL", str(len(data)), str(len(ids_dict['M'])+len(ids_dict['F'])+len(ids_dict['U']))] + get_durations_from_data(data))
        output_file.write(line + "\n")

def get_stats(file_name):
    #input_file = ("./metadata/"+file_name)
    input_file = (file_name)
    data = load_json(input_file)
    print_stats(data)
    write_stats(data, input_file.replace(".json","_stats.csv"))

if __name__ == "__main__":
    get_stats(sys.argv[1])
    