#!/usr/bin/python3
"""
Preparation of the partitioning of jsons about audio data

Author: Soline Felice

Date: 19/10/2022

Goal : Create a json file with filtered informations

Input :
- ./metadata/downloaded.csv (informations about the audios extract at the same time)
- ./metadata/ids_gender.txt (speakers ID and gender)
- ./wavs/example.wav (the audios)

Treatment : 
- Search into the data load from the downloaded.csv :
    - Data where the category of books aren't "charme" (erotic), "planete-actuelle" (geopolitic) or "religion" (the filtered data)
- Get this data and write it into a new json file : "all_filtered_files.json"

Output :
- ./metadata/all_filtered_files.json

Prerequisites :
- Libraries installed :
    - pydub

- Architecture (folders and files necessary):
    - ./scripts/
        - prep_json_filter.py (this script)
        - funcs.py (necessary)

    - ./metadata/
        - ids_gender.txt (file with the informations of the speakers and their genders in format "gender\tname", ex : F\tCarole \n M\tGérard))
        - downloaded.csv (informations about the audios extract at the same time)

    - ./wavs/
        - (wavs folders with the audios files in it)

"""

import os, glob, math
from funcs import get_files_in_path, printProgressBar
from pydub import AudioSegment
from subprocess import Popen, PIPE
import subprocess
import signal
import time
import requests
import html
import json
import re
import copy
import wave
import ffmpeg
import csv

#########

# output json file about the complet audio and directly integrating the gender informations
output_file=("../metadata/all_filtered_files.json")

# text file with informations of gender
spk_info_filename=("../metadata/ids_gender.txt")

#########
class FormatString():

    def __init__(self,filename, folderpath, tmpFolder):

        self.filename = filename
        self.folderpath = folderpath
        self.tmpFolder = tmpFolder
        self.changedName = '1a_changed_filename.log'
        self.changeNameError = '1b_changeNameError.log'

    def format_string(self):

        oldName = copy.deepcopy(self.filename)
        newFilename = re.sub('[^a-zA-Z0-9\n\.]','_',self.filename)

        if oldName != newFilename :
            oldPath = self.folderpath + '/' + oldName
            newPath = self.folderpath + '/' + newFilename

            try:
                os.rename(oldPath,newPath)
                with open(self.tmpFolder + "/" + self.changedName, mode='a', encoding='utf-8') as catch_changeName:
                    catch_changeName.write('Filename {} changed to {} \n'.format(oldPath,newPath))
            
            except PermissionError:
                with open(self.tmpFolder + "/" + self.changeNameError, mode='a', encoding='utf-8', errors='namereplace') as catch_changeName:
                    catch_changeName.write('Filename {} couldn\'t be changed to {} because of permissionError \n'.format(oldPath,newPath))
            
            except OSError as error:
                with open(self.tmpFolder + "/" + self.changeNameError, mode='a', encoding='utf-8', errors='namereplace') as catch_changeName:
                    catch_changeName.write('Filename {} couldn\'t be changed to {} because of OSError \n'.format(oldPath, newPath))

            except :
                with open(self.tmpFolder + "/" + self.changeNameError, mode='a', encoding='utf-8', errors='namereplace') as catch_changeName:
                    catch_changeName.write('Filename {} couldn\'t be changed to {} because of some error \n'.format(oldPath, newPath))

        return newFilename

class FormatAudio():

    def __init__(self, folderPath, folder_name, tmpFolder, filename, savepath):

        self.folderPath = folderPath
        self.folder_name = folder_name
        self.tmpFolder = tmpFolder 
        self.filename = filename
        self.filepath = self.folderPath + '/' + self.filename
        self.filename_no_extension = os.path.splitext(self.filename)[0]
        self.savepath = savepath + '/' + self.folder_name
        self.errorFile = '2_audio_not_converted.log'

    def audio_to_16k_wav(self):
        self.cmd="sox -S " + self.filepath + " -r 16000 -c 1 -b 16 -e signed-integer '" + self.savepath + "/" + self.filename_no_extension + ".mp3' 2>&1 | grep 'sox FAIL formats' >> " + self.tmpFolder + self.errorFile 
        print("Conversion de "+self.filepath)

        timeout = 1200

        try:
            self.process = subprocess.call(self.cmd, stdout=None, stderr=None, shell=True, cwd=None, timeout=timeout) #timeout in seconds

        except subprocess.TimeoutExpired:   
            with open(self.tmpFolder + "/" + self.errorFile, mode='a', encoding='utf-8') as catch_error:
                catch_error.write('\n!ERROR Sox terminated after {} seconds, executing {} \n'.format(timeout,self.cmd))
    
class CreateJson():

    def __init__(self, folderPath, folder_name, filename, metadataDict, outputFolder,json_file_num, keys_list, i):

        self.folderPath = folderPath + '/'
        self.folder_name = folder_name
        self.filename = filename
        self.filepath = self.folderPath + self.filename
        self.audio = AudioSegment.from_wav(self.filepath)
        self.metadataDict = metadataDict
        self.outputFolder = outputFolder
        self.json_file_num = json_file_num
        self.jsonFileName = "all_filtered_files.json"
        self.csvFileName = "final_CSV.csv"
        self.keys_list = keys_list
        self.i = i

    # 4. Duration in seconds
    def get_duration(self):
        wavDuration = self.audio.duration_seconds
        return wavDuration
     
    # 5. Speaker id
    def get_spk_id(self):

        # Getting the directory of the wavs
        dir = os.path.dirname(os.path.dirname(self.filepath)) # dir of dir of file
        dirname1 = os.path.basename(dir)

        wavNameWithExtension = re.sub(f"(_[0-9]*).mp3","",self.filename)
        wavName = os.path.splitext(wavNameWithExtension)[0]
        if self.filename != wavNameWithExtension :
            wavName = os.path.splitext(self.filename)[0]

        if wavName not in self.metadataDict :
            ### little trick to get and put the name of the folder (not extracted) as the wav name if this one isn't in metadataDict ###
            #print("Name of the folder : ", dirname1)
            wavName = dirname1
           
        try:
            spk_id = self.metadataDict[wavName]['speaker']
            self.write_final_CSV(wavName)
            # To uniformize (the "UNK" into "U")
            if spk_id == "UNK" :
                spk_id = "U"
            return spk_id           
            
        except KeyError as e:
            with open(self.outputFolder+'keysError.txt',"a+") as keyErrFile:
                keyErrFile.write("key not found "+str(e)+" in metadataDict \n")

    # 6. Speaker gender
    def get_spk_gender(self,spk_id):

        spk_info_file=open(spk_info_filename,mode="r")

        wavNameWithExtension = re.sub(f"(_[0-9]*).mp3","",self.filename)
        wavName = os.path.splitext(wavNameWithExtension)[0]
        if self.filename != wavNameWithExtension :
            wavName = os.path.splitext(self.filename)[0]

        if wavName not in self.metadataDict :
            ### little trick to get and put the name of the folder (not extracted) as the wav name if this one isn't in metadataDict ###
            dir = os.path.dirname(os.path.dirname(self.filepath))
            dirname1 = os.path.basename(dir)
            wavName = dirname1

        num_keys=0

        id_genre={}

        try:
            spk_id = self.metadataDict[wavName]['speaker']
            for line in spk_info_file:
                line=line.split('\t')
                spk_id_txt=line[0].strip()
                gender_txt=line[1].strip()
                
                if spk_id_txt in spk_id :
                    spk_gender = gender_txt
                    return spk_gender
                    

        except KeyError as e:
            with open(self.outputFolder+'keysError.txt',"a+") as keyErrFile:
                keyErrFile.write("key not found "+str(e)+" in metadataDict \n")

    def create_json_entry(self, wavDuration, spk_id, spk_gender):
        d = {}
        d["path"] = self.filepath
        d["trans"] = ""
        d["duration"] = wavDuration
        d["spk_id"] = spk_id
        d["spk_gender"] = spk_gender
        return d
        


    def create_dict_to_dump(self,new_entry,json_data): 
        """! Warning: if two files have the same name, they won't be added to the dict twice. The second will replace the first. The if statement is needed."""   

        wavNameWithExtension = re.sub(f"(_[0-9]*).wav","",self.filename)
        wavName = os.path.splitext(wavNameWithExtension)[0]
        if self.filename != wavNameWithExtension :
            wavName = os.path.splitext(self.filename)[0]

        if wavName not in self.metadataDict :
            ### little trick to get and put the name of the folder (not extracted) as the wav name if this one isn't in metadataDict ###
            #print("Name of the folder : ", dirname1)
            dir = os.path.dirname(os.path.dirname(self.filepath))
            dirname1 = os.path.basename(dir)
            wavName = dirname1

        if self.filename in json_data:
            self.filename = self.filename+'_'+str(self.i)

        ### here we get the category to remove the files with the categories "charme" (erotic audio) or "planete-actuelle" (geopolitical audio) or "religions" ###
        doc_cat = self.metadataDict[wavName]['category']
        if doc_cat == "charme" or doc_cat == "planete-actuelle" or doc_cat == "religions" : 
            pass
        else :
            json_data[self.filename] = new_entry 
        return json_data

    def write_json_file(self,json_data):
        # write dict and dump. Creates multiple json (mandatory because dumping dict too large fails and data needs to be written once
        # or it creates multiple json objects and leads to a badly formed json file)

        print("\nJson file is being written \n")
        with open(self.outputFolder + self.jsonFileName, mode='a', encoding='utf-8') as output_file:
            json.dump(json_data, output_file, ensure_ascii=False, indent=2, separators=(',', ': '))
        json_data.clear() 

    def write_final_CSV(self, wavName):
        with open(self.outputFolder + self.csvFileName, mode='a', encoding='utf-8') as final_csv:
            for key,value in self.metadataDict.items():
                if key == wavName:
                    if key not in self.keys_list:
                        writer = csv.writer(final_csv, delimiter =";",quoting=csv.QUOTE_MINIMAL)
                        value['license']=re.sub('"','',value['license'])
                        value['license']=re.sub(',','',value['license']) 
                        writer.writerow([value['status'],value['title'],value['audio_url'],value['author'],value['category'],value['license'],value['speaker'],value['book_url'],value['download_folder']])
                        self.keys_list.append(key)
                    else:
                        break

def main():
    wavsPath = "../wavs/" #Not converted files
    wavsConvertedPath = "../wavs/" #Converted files
    savPath = "../wavs/" #Converted + split files. Saves in $savPath + '/*/split/', * being the audio_folder
    metadataCSV_path = "../metadata/downloaded.csv"
    outputFolder = "../metadata/" #for json file
    tmpFolder = "../tmp/"
    spk_info_filename = "../metadata/ids_gender.txt"
    extension = "mp3"

    ###### Create output dirs

    if not os.path.isdir(outputFolder):
        os.mkdir(outputFolder)

    if not os.path.isdir(tmpFolder): # files.log
        os.mkdir(tmpFolder)


    
    ###### Convert files
    theFiles=[]

    for audio_folder in os.listdir(wavsPath):
        if os.path.isdir(wavsPath + audio_folder + '/extracted'):
            audio_folder = audio_folder + '/extracted/'
            theFiles.append(get_files_in_path(wavsPath, audio_folder, extension)) 
        else:
            theFiles.append(get_files_in_path(wavsPath, audio_folder, extension)) 
    num_Files = len(theFiles)

    ###### Read CSV and create dict 

    metadataDict = {}
    metadataSubDict = {}


    ###### Open of downloaded.csv
    with open(metadataCSV_path) as CSVFile:
        next(CSVFile)
        for line in CSVFile:

            column = line.split(";")
            title = column[0]
            audio_url = column[1]
            author = column[2]
            category = column[3]
            license = column[4]
            speaker = column[5]
            book_url = column[6]
            download_folder = column[7]

            metadataSubDict['title'] = title.strip("\n")
            metadataSubDict['audio_url'] = audio_url.strip("\n")
            metadataSubDict['author'] = author.strip("\n")
            metadataSubDict['category'] = category.strip("\n")
            metadataSubDict['license'] = license.strip("\n")
            metadataSubDict['speaker'] = speaker.strip("\n")
            metadataSubDict['book_url'] = book_url.strip("\n")
            metadataSubDict['download_folder'] = download_folder.strip("\n")
            
            wavName = download_folder[5:] #get the download folder name which is the same as audiofile name
            wavName = wavName.rstrip('/\n')
            metadataDict[wavName] = dict(metadataSubDict)

    ###### Create json

    convertedWavs=[]
    keys_already_in_csv = []

    for audio_folder in os.listdir(savPath):
        if os.path.isdir(savPath + audio_folder + '/extracted'):
            audio_folder = audio_folder + '/extracted/'
            convertedWavs.append(get_files_in_path(savPath, audio_folder, "mp3"))
        else:
            audio_folder = audio_folder
            convertedWavs.append(get_files_in_path(savPath, audio_folder, "mp3"))

    entry_num=0
    json_file_num=1
    json_data={}
    i=0


    for i, filePath in enumerate(convertedWavs):
        for j, file_path in enumerate(filePath):
            folderPath = os.path.dirname(file_path)
            file = os.path.basename(file_path)
            if folderPath[-16:] == '/extracted' :
                folder_name = folderPath[:-16]
            else:
                folder_name = folderPath[:-6]
            folder_name = os.path.split(folder_name)[1]
            json_file = CreateJson(folderPath, folder_name, file, metadataDict, outputFolder,json_file_num, keys_already_in_csv, i)
            wav_duration = json_file.get_duration()
            wav_spk_id = json_file.get_spk_id()
            wav_spk_gender = json_file.get_spk_gender(wav_spk_id)
            new_entry = json_file.create_json_entry(wav_duration, wav_spk_id, wav_spk_gender) #wav_spk_gender
            #json_file.write_final_CSV()

            if  entry_num <=40000: #sets number of json entries in json file. If too many, it crashes. Default = 40 000
                dict_to_dump = json_file.create_dict_to_dump(new_entry,json_data)
                entry_num+=1

            else:
                dict_to_dump = json_file.create_dict_to_dump(new_entry,json_data)
                json_file.write_json_file(dict_to_dump)
                entry_num=0
                json_file_num+=1

    json_file_num+=1
    json_file.write_json_file(dict_to_dump)

if __name__== "__main__":

    main()                 